@@ -1,6 +1,6 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2017, 2018, 2019, 2020, 2021, 2023 Janneke Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2017, 2018, 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2017, 2018, 2019 Rob Wieringa <rma.wieringa@gmail.com>
 ;;; Copyright © 2017, 2021 Rutger van Beusekom <rutger@dezyne.org>
 ;;;
@@ -26,12 +26,10 @@
 (define-module (dzn commands code)
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-26)
-  #:use-module (srfi srfi-71)
   #:use-module (ice-9 getopt-long)
   #:use-module (ice-9 poe)
   #:use-module (dzn config)
   #:use-module (dzn code)
-  #:use-module (dzn misc)
   #:use-module (dzn shell-util)
   #:use-module (dzn command-line)
   #:use-module (dzn commands parse)
@@ -41,14 +39,11 @@
 
 (define %default-language "c++")
 (define (list-languages dir)
-  (let* ((uninstalled? (getenv "DZN_UNINSTALLED"))
-         (ext path (if uninstalled? (values ".scm" %load-path)
-                       (values ".go" %load-compiled-path)))
-         (regex (string-append "\\" ext "$")))
-    (map (cut basename <> ext)
-         (filter (cute string-contains <> dir)
-                 (append-map (cute find-files <> regex)
-                             (filter directory-exists? path))))))
+  (map (cut basename <> ".go")
+       (filter (cute string-contains <> dir)
+               (append-map (cute find-files <> "\\.go$")
+                           (filter directory-exists?
+                                   %load-compiled-path)))))
 
 (define list-languages (pure-funcq list-languages))
 
@@ -65,6 +60,7 @@
             (language (single-char #\l) (value #t))
             (locations (single-char #\L))
             (model (single-char #\m) (value #t))
+            (no-blocking (single-char #\B))
             (output (single-char #\o) (value #t))
             (queue-size (single-char #\q) (value #t))
             (shell (single-char #\s) (value #t))))
@@ -85,6 +81,7 @@ Generate code for Dezyne models in DZN-FILE
   -l, --language=LANG         generate code for language=LANG [~a]
   -L, --locations             prepend locations to output trace
   -m, --model=MODEL           generate main for MODEL
+  -B, --no-blocking           assume system without collateral blocking
   -o, --output=DIR            write output to DIR (use - for stdout)
   -q, --queue-size=SIZE       use queue size SIZE
   -s, --shell=MODEL           generate thread safe system shell for MODEL
@@ -103,7 +100,7 @@ Languages: ~a
          (language (option-ref options 'language %default-language))
          (locations? (option-ref options 'locations #f))
          (model (option-ref options 'model #f))
-         (queue-size (option-ref options 'queue-size 3))
+         (queue-size (command-line:get-number 'queue-size 3))
          (shell (option-ref options 'shell #f))
          ;; Parse --model=MODEL cuts MODEL from AST; avoid that
          (parse-options (filter (negate (compose (cut eq? <> 'model) car)) options))
