@@ -1,6 +1,6 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2019, 2020, 2021 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2021 Rutger van Beusekom <rutger@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
@@ -45,8 +45,11 @@
             (internal (single-char #\i))
             (locations (single-char #\l))
             (model (single-char #\m) (value #t))
+            (no-compliance (single-char #\C))
             (no-deadlock (single-char #\D))
             (no-interface-determinism)
+            (no-interface-livelock)
+            (no-queue-full (single-char #\Q))
             (no-refusals (single-char #\R))
             (queue-size (single-char #\q) (value #t))
             (state (single-char #\s))
@@ -63,7 +66,9 @@
 Usage: dzn simulate [OPTION]... [FILE]...
 Simulate a Dezyne model
 
+  -C, --no-compliance    skip the compliance check
   -D, --no-deadlock      skip the deadlock check
+  -Q, --no-queue-full    skip the external queue-full check
   -R, --no-refusals      skip the refusals check
   -f, --format=FORMAT    display trace in format FORMAT [event] {diagram,event,trace}
   -h, --help             display this help and exit
@@ -74,6 +79,8 @@ Simulate a Dezyne model
   -m, --model=MODEL      generate main for MODEL
       --no-interface-determinism
                          skip interface RTC determinism check
+      --no-interface-livelock
+                         skip interface livelock check at EOT
   -q, --queue-size=SIZE  use queue size=SIZE for simulation [3]
       --state            show state after every action, trigger
   -s, --strict           use strict matching of trail
@@ -92,12 +99,15 @@ Simulate a Dezyne model
          ;; Parse --model=MODEL cuts MODEL from AST; avoid that
          (parse-options (filter (negate (compose (cut eq? <> 'model) car)) options))
          (ast (parse parse-options file-name))
+         (no-compliance? (option-ref options 'no-compliance #f))
          (no-deadlock? (option-ref options 'no-deadlock #f))
          (no-interface-determinism?
           (option-ref options 'no-interface-determinism #f))
+         (no-interface-livelock?
+          (option-ref options 'no-interface-livelock #f))
+         (no-queue-full? (option-ref options 'no-queue-full #f))
          (no-refusals? (option-ref options 'no-refusals #f))
-         (queue-size (command-line:get 'queue-size "3"))
-         (queue-size (string->number queue-size))
+         (queue-size (command-line:get-number 'queue-size 3))
          (state? (command-line:get 'state #f))
          (strict? (command-line:get 'strict #f))
          (verbose? (command-line:get 'verbose #f))
@@ -107,9 +117,13 @@ Simulate a Dezyne model
          (trail (option-ref options 'trail #f))
          (status (simulate ast
                            #:model-name model-name
+                           #:compliance-check? (not no-compliance?)
                            #:deadlock-check? (not no-deadlock?)
                            #:interface-determinism-check?
                            (not no-interface-determinism?)
+                           #:interface-livelock-check?
+                           (not no-interface-livelock?)
+                           #:queue-full-check? (not no-queue-full?)
                            #:refusals-check? (not no-refusals?)
                            #:internal? internal?
                            #:locations? locations?
