@@ -1,8 +1,8 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2020, 2021 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2020, 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2021 Paul Hoogendijk <paul@dezyne.org>
-;;; Copyright © 2021 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -79,10 +79,15 @@
                         (disjoin (compose null? .async car)
                                  did-provides-out?
                                  (compose .status car))
-                        traces)))
-            (append stop (append-map (cute flush-async-trace <> '()) flush)))))
+                        traces))
+                 (port (.port (string->trigger event)))
+                 (blocked-port (and=> (blocked-on-boundary? pc) .ast)))
+            (if (and port (ast:eq? port blocked-port)) '()
+                (append stop (append-map (cute flush-async-trace <> '())
+                                         flush))))))
      ((provides-trigger? event)
-      (run-to-completion pc event)))))
+      (if (blocked-on-boundary-provides? pc event) '()
+          (run-to-completion pc event))))))
 
 ;; table is initially (<"<illegal>") . 0>)
 (define* ((pc->state-number table count) pc #:optional trail)
@@ -108,11 +113,64 @@ recursion.  Return a run-to-completion LTS
 
   (LTS PC->STATE-NUMBER AND STATE-NUMBER-COUNT)
 "
+  (define (collateral? pc previous-pc)
+    "Return #true if the only difference between PC and PREVIOUS-PC is
+that PC has one more collaterally blocked coroutine on the same port."
+    (define (drop-collateral pc)
+      (match (.collateral pc)
+        (((port . p) t ...)
+         (clone pc
+                #:collateral t
+                #:previous (and=> (.previous pc) drop-collateral)))
+        (_
+         pc)))
+    (match (.collateral pc)
+      (((port . pc0) t ...)
+       (and (find (compose (cute eq? <> (.instance pc0)) .instance cdr) t)
+            (let ((pc (drop-collateral pc)))
+              (pc-equal? pc previous-pc))))
+      (_
+       #f)))
+
+  (define (run-label orig-pc label)
+    (let* ((pc (switch-context orig-pc))
+           (switched? (not (eq? pc orig-pc)))
+           (pc (if switched? pc
+                   (blocked-on-boundary-switch-context pc)))
+           (bop-switched? (and (not switched?)
+                               (not (eq? pc orig-pc))))
+           (pc (if (or switched? bop-switched?) pc
+                   (blocked-on-boundary-collateral-release pc))))
+      (if (eq? pc orig-pc) (run-to-completion** pc label)
+          (if (and switched?
+                   (not (provides-trigger? label)))
+              (run-to-completion pc 'rtc)
+              (append (run-to-completion pc 'rtc)
+                      (run-to-completion** orig-pc label))))))
+
+  (define (run-labels pc labels)
+    (let loop ((labels labels) (traces '()))
+      (if (null? labels) traces
+          (let* ((label (car labels))
+                 (new (run-label pc label))
+                 (new (if (eq? label 'external) new
+                          (filter-implicit-illegal-only new)))
+                 (new (filter (compose not (is-status? <blocked-error>) car)
+                              new))
+                 (new (filter (compose not (cute collateral? <> pc) car) new)))
+            (cond
+             ((find (compose (is-status? <livelock-error>) car) new)
+              (format (current-error-port) "warning: livelock, bailing out\n")
+              '())
+             (else
+              (loop (cdr labels) (append new traces))))))))
+
   (let* ((labels (cons* 'async 'external (labels)))
          (lts (make-hash-table))
          (state-number-table (make-hash-table))
          (state-number-count (list 1))
-         (pc->state-number (pc->state-number state-number-table state-number-count)))
+         (pc->state-number (pc->state-number state-number-table
+                                             state-number-count)))
     (hash-set! state-number-table "<illegal>" 0)
     (when (.status pc)
       (let ((pc0 (clone pc #:status #f)))
@@ -120,19 +178,12 @@ recursion.  Return a run-to-completion LTS
     (let loop ((pc pc))
       (let ((from (pc->state-number pc)))
         (unless (or (.status pc) (hash-ref lts from))
-          (let* ((pc (clone pc #:instance #f #:trail '()))
-                 (traces
-                  (let loop ((labels labels) (traces '()))
-                    (if (null? labels) traces
-                        (let* ((new (run-to-completion** pc (car labels)))
-                               (new (filter-implicit-illegal-only new)))
-                          (cond ((find (compose (is-status? <livelock-error>) car) new)
-                                 (format (current-error-port) "warning: livelock, bailing out\n")
-                                 '())
-                                (else
-                                 (loop (cdr labels) (append new traces))))))))
+          (let* ((from-pc pc)
+                 (pc (clone pc #:collateral-instance #f))
+                 (traces (run-labels pc labels))
+                 (traces (map (cute append <> (list from-pc)) traces))
                  (pcs (map car traces)))
-            (hash-set! lts from (cons pc traces))
+            (hash-set! lts from (cons from-pc traces))
             (let* ((traces (filter (negate trace-done?) traces))
                    (pcs (map car traces)))
               (for-each loop pcs))))))
@@ -140,7 +191,53 @@ recursion.  Return a run-to-completion LTS
       (hash-set! lts 1 (cons pc (list (list pc)))))
     (values lts pc->state-number state-number-count)))
 
-(define* (lts-remove lts size #:key ports? extended? actions? labels?
+
+;;;
+;;; State diagram
+;;;
+(define (rtc-lts->state-diagram lts pc->state-number)
+  "Create a state-diagram from run-to-completion LTS, return
+
+   ((from from-label transition to trigger-location)
+    ...)
+"
+  (define (trigger-location trace)
+    (let ((pc (find (conjoin (compose (is? <on>) .statement)
+                             (disjoin (const (is-a? (%sut) <runtime:port>))
+                                      (compose (is? <runtime:component>) .instance)))
+                    (reverse trace))))
+      (and pc (and=> (.statement pc) ast:location))))
+
+  (define (trace->string-trail trace)
+    (let ((trail (map cdr (trace->trail trace))))
+      (define (strip-sut-prefix o)
+        (if (string-prefix? "sut." o) (substring o 4) o))
+      (map strip-sut-prefix trail)))
+
+  (define ((transition->dot pc->state-number) from pc+traces)
+    (let* ((pc traces (match pc+traces ((pc . traces) (values pc traces))))
+           (traces (filter (compose not .status car) traces))
+           (pcs (map car traces))
+           (transition (map trace->string-trail traces))
+           (to (map pc->state-number pcs))
+           (to+transition (delete-duplicates
+                           (map cons to transition)
+                           (match-lambda*
+                             (((a-to a-transition ...) (b-to b-transition ...))
+                              (and (= a-to b-to)
+                                   (equal? a-transition b-transition))))))
+           (to (map car to+transition))
+           (transition (map cdr to+transition))
+           (trigger-location (map trigger-location traces)))
+      (if (null? to) (list (list from pc #f #f #f))
+          (map (cut list from pc <> <> <>)
+               transition
+               to
+               trigger-location))))
+
+  (apply append (hash-map->list (transition->dot pc->state-number) lts)))
+
+(define* (lts-remove lts size #:key ports? extended? actions? labels? returns?
                      (self? #t))
   "Remove from the LTS transitions which only differ in terms of
 EXTENDED?, PORTS?, LABELS? or ACTIONS?  When SELF?, remove all self
@@ -177,11 +274,20 @@ transitions."
                (state (make <system-state> #:state-list state-list)))
           (clone pc #:state state))))
 
-  (define (hide-return-value pc)
+  (define (action-return? pc)
     (let ((statement (.statement pc)))
-      (if (is-a? statement <trigger-return>)
-          (clone pc #:statement (clone statement #:event.name "return"))
-          pc)))
+      (and (is-a? statement <trigger-return>)
+           (not (.port statement))
+           (ast:requires? (.instance pc)))))
+
+  (define (hide-return pc)
+    (let ((statement (.statement pc)))
+      (and (not (and (action-return? pc)
+                     (equal? (.event.name statement) "return")))
+           pc)))
+
+  (define (hide-returns trace)
+    (filter-map hide-return trace))
 
   (define hide-actions
     (match-lambda
@@ -189,8 +295,10 @@ transitions."
        (let* ((reversed (reverse trace))
               (trigger (any .trigger reversed))
               (action (find (compose (is? <action>) .statement) reversed))
-              (trace (filter (compose not (is? <action>) .statement) trace))
-              (trace (map hide-return-value trace))
+              (trace (filter (negate (disjoin
+                                      (compose (is? <action>) .statement)
+                                      action-return?))
+                             trace))
               (trace (if (ast:modeling? trigger)
                          (reverse (cons action (reverse trace)))
                          trace)))
@@ -224,6 +332,9 @@ transitions."
                    (traces
                     (if (or labels? (not actions?)) traces
                         (map hide-actions traces)))
+                   (traces
+                    (if (or labels? (not returns?)) traces
+                        (map hide-returns traces)))
                    (traces (if (not labels?) traces
                                (map (compose list car) traces))))
               (hash-set! result from (cons pc traces))
@@ -233,53 +344,6 @@ transitions."
             pc->state-number
             state-number-count)))
 
-
-;;;
-;;; State diagram
-;;;
-(define (rtc-lts->state-diagram lts pc->state-number)
-  "Create a state-diagram from run-to-completion LTS, return
-
-   ((from from-label transition to trigger-location)
-    ...)
-"
-  (define (trigger-location trace)
-    (let ((pc (find (conjoin (compose (is? <on>) .statement)
-                             (disjoin (const (is-a? (%sut) <runtime:port>))
-                                      (compose (is? <runtime:component>) .instance)))
-                    (reverse trace))))
-      (and pc (and=> (.statement pc) ast:location))))
-
-  (define (trace->string-trail trace)
-    (let* ((trail (map cdr (trace->trail trace)))
-           (trail (filter (negate (cute string-suffix? ".return" <>)) trail)))
-      (define (strip-sut-prefix o)
-        (if (string-prefix? "sut." o) (substring o 4) o))
-      (map strip-sut-prefix trail)))
-
-  (define ((transition->dot pc->state-number) from pc+traces)
-    (let* ((pc traces (match pc+traces ((pc . traces) (values pc traces))))
-           (traces (filter (compose not .status car) traces))
-           (pcs (map car traces))
-           (transition (map trace->string-trail traces))
-           (to (map pc->state-number pcs))
-           (to+transition (delete-duplicates
-                           (map cons to transition)
-                           (match-lambda*
-                             (((a-to a-transition ...) (b-to b-transition ...))
-                              (and (= a-to b-to)
-                                   (equal? a-transition b-transition))))))
-           (to (map car to+transition))
-           (transition (map cdr to+transition))
-           (trigger-location (map trigger-location traces)))
-      (if (null? to) (list (list from pc #f #f #f))
-          (map (cut list from pc <> <> <>)
-               transition
-               to
-               trigger-location))))
-
-  (apply append (hash-map->list (transition->dot pc->state-number) lts)))
-
 (define (state-diagram->dot graph start)
   "Return a diagram in DOT format from GRAPH produced by
 RTC-LTS->STATE-DIAGRAM."
@@ -427,7 +491,8 @@ numbers"
     "From a run-to-completion transition PC,TRACE, generate one LTS
 triple per label in the trail of the transition."
     (let* ((trace (fix-missing-reply-trace trace))
-           (labels (map cdr (trace->trail trace)))
+           (labels (parameterize ((%modeling? #t))
+                     (map cdr (trace->trail trace))))
            (labels (if (null? labels) '("tau") labels))
            (to (pc->state-number (car trace)))
            (inner-state-numbers (trail->state-numbers pc labels to)))
@@ -461,26 +526,24 @@ RTC-LTS->LTS."
 ;;;
 
 (define* (state-diagram root #:key format model queue-size
-                        ports? extended? actions? labels?)
+                        ports? extended? actions? labels? returns?)
   "Entry-point for dzn explore --state-diagram."
-  (when (> (dzn:debugity) 0)
-    (set! %debug? #t))
-  (parameterize ((%exploring? #t)
+  (parameterize ((%debug? (> (dzn:debugity) 0))
+                 (%exploring? #t)
                  (%queue-size queue-size)
                  (%sut (runtime:get-sut root model)))
     (parameterize ((%instances (runtime:create-instances (%sut))))
       (let* ((pc (make-pc))
              (lts pc->state-number state-count (pc->rtc-lts pc))
-             (remove? (or ports? extended? actions? labels?))
              (size (1- (car state-count)))
              (lts pc->state-number state-count
-                  (if remove? (lts-remove lts size
-                                          #:ports? (or ports? extended?)
-                                          #:extended? extended?
-                                          #:actions? actions?
-                                          #:labels? labels?
-                                          #:self? #t)
-                      (values lts pc->state-number state-count)))
+                  (lts-remove lts size
+                              #:ports? (or ports? extended?)
+                              #:extended? extended?
+                              #:actions? actions?
+                              #:labels? labels?
+                              #:returns? returns?
+                              #:self? (or extended? ports?)))
              (state-diagram (rtc-lts->state-diagram lts pc->state-number)))
         (if (equal? format "json") (display
                                     (state-diagram->json
@@ -489,9 +552,8 @@ RTC-LTS->LTS."
 
 (define* (lts root #:key model queue-size)
   "Entry-point for dzn explore --lts."
-  (when (> (dzn:debugity) 0)
-    (set! %debug? #t))
-  (parameterize ((%exploring? #t)
+  (parameterize ((%debug? (> (dzn:debugity) 0))
+                 (%exploring? #t)
                  (%queue-size queue-size)
                  (%sut (runtime:get-sut root model)))
     (parameterize ((%instances (runtime:create-instances (%sut))))
