@@ -610,6 +610,29 @@ Add (synthesize) missing PCs for <q-in>, <q-out> and <trigger-return>."
                    (q-in (clone q-in #:parent (.ast deferred)))
                    (q-pc (clone pc #:instance deferred #:statement q-in)))
               (loop (cdr trace) (cons* pc q-pc result))))
+           ((and (is-a? statement <action>)
+                 (ast:out? statement)
+                 (is-a? pc-instance <runtime:component>)
+                 (or (not next)
+                     (not (is-a? (.statement next) <action>))
+                     (not (equal? (.event.name (.statement next))
+                                  (.event.name statement)))
+                     (let* ((port (.port statement))
+                            (r:port (runtime:port pc-instance port))
+                            (r:other-port (runtime:other-port r:port)))
+                       (not (eq? r:other-port (.instance next))))))
+            (let* ((port (.port statement))
+                   (r:port (runtime:port pc-instance port))
+                   (r:other-port (and r:port (runtime:other-port r:port)))
+                   (interface (.type port))
+                   (action (clone statement
+                                  #:port.name #f
+                                  #:location (.location statement)))
+                   (action (clone action #:parent interface))
+                   (action-pc (clone pc
+                                     #:instance r:other-port
+                                     #:statement action)))
+              (loop (cdr trace) (cons* pc action-pc result))))
            ((and (is-a? statement <q-out>)
                  (is-a? pc-instance <runtime:component>))
             (let* ((trigger (.trigger statement))
